@@ -15,41 +15,38 @@ use Data::Dumper;
 use Moose;
 extends 'Connector::Builtin';
 
-sub _build_config {
-    my $self = shift;
-
-    if (! -r $self->{LOCATION}) {
-	   confess("Cannot open input file " . $self->{LOCATION} . " for reading.");
-    }
-    
-    return 1;
-}
-
-
 sub get {
-    
+
     my $self = shift;
-    my $arg = shift;
 
     my $filename = $self->{LOCATION};
 
-    my $content;
-    if (-r $filename) {
-	$content = do {
-	  local $INPUT_RECORD_SEPARATOR;
-	  open my $fh, '<', $filename;
-	  <$fh>;
-      };
+    if (! -r $filename) {
+        return $self->_node_not_exists( );
     }
 
+    my $content = do {
+      local $INPUT_RECORD_SEPARATOR;
+      open my $fh, '<', $filename;
+      <$fh>;
+    };
+
     return $content;
 }
 
-sub get_meta {    
+sub get_meta {
     my $self = shift;
-    return {TYPE  => "scalar", VALUE => $self->get() };    
-}   
+    return { TYPE  => "scalar" };
+}
 
+sub exists {
+
+    my $self = shift;
+
+    my $filename = $self->{LOCATION};
+
+    return -r $filename;
+}
 no Moose;
 __PACKAGE__->meta->make_immutable;
 
@@ -64,4 +61,3 @@ Connector::Builtin::File::Simple
 
 Return the contents of the file given by the LOCATION parameter.
 The path argument is discarded.
-
